@@ -70,6 +70,9 @@
 #define _VER_(major, minor, patchlevel) \
 	((major * 10000) + (minor * 100) + patchlevel)
 
+#define FLOAT_TINY		(0.0000001)
+#define FLOAT_CMP(a, b)		(fabs(a - b) < FLOAT_TINY)
+
 #if defined(__GNUC__) && defined(__GNUC_MINOR__)
 #if defined(__GNUC_PATCHLEVEL__)
 #define NEED_GNUC(major, minor, patchlevel) \
@@ -515,18 +518,30 @@ static char *get_pid_cmdline(const pid_t pid)
 
 no_cmd:
 	if (statok) {
+		bool new_info = false;
 		/*
 		 * We may be re-using a stale old PID, or we may need
 		 * a new info struct 
 		 */
-		if (!info)
+		if (!info) {
 			info = malloc(sizeof(pid_info_t));
+			new_info = true;
+		}
+
+		/*
+		 *  Don't worry if we can't allocate cache info
+		 *  as we can fetch it next time if need be
+		 */
 		if (LIKELY((info != NULL))) {
 			info->pid = pid;
 			info->cmdline = ptr;
-			info->next = pid_info_hash[h];
 			info->st_ctim = statbuf.st_ctim;
-			pid_info_hash[h] = info;
+
+			/* Only append to hash list if new */
+			if (new_info) {
+				info->next = pid_info_hash[h];
+				pid_info_hash[h] = info;
+			}
 		}
 	}
 	return ptr;
@@ -588,7 +603,7 @@ static void OPTIMIZE3 HOT sample_add(
 	sample_delta_item_t *sdi;
 
 	for (sdl = sample_delta_list_tail; sdl; sdl = sdl->prev) {
-		if (sdl->whence == whence) {
+		if (FLOAT_CMP(sdl->whence, whence)) {
 			found = true;
 			break;
 		}
@@ -653,8 +668,8 @@ static inline OPTIMIZE3 HOT sample_delta_item_t *sample_find(
  */
 static int info_compare_total(const void *const item1, const void *const item2)
 {
-	cpu_info_t **info1 = (cpu_info_t **)item1;
-	cpu_info_t **info2 = (cpu_info_t **)item2;
+	cpu_info_t *const *info1 = (cpu_info_t *const *)item1;
+	cpu_info_t *const *info2 = (cpu_info_t *const *)item2;
 
 	if ((*info2)->total == (*info1)->total)
 		return 0;
@@ -841,10 +856,10 @@ static void samples_dump(
 			sample_delta_item_t *sdi =
 				sample_find(sdl, sorted_cpu_infos[i]);
 			if (sdi) {
-				double duration =
+				double tmp_duration =
 					duration_round(sdi->time_delta);
 				fprintf(fp,",%f",
-					(duration < 0.01) ? 0.0 :
+					(tmp_duration < 0.01) ? 0.0 :
 					100.0 * (double)sdi->delta /
 					(duration * (double)nr_ticks));
 			} else
